/********************* rexx ****************************************/
/*     (c)  Copyright   Roger Lacroix  1993                         |
|                                                                   |
|  Designed, developed and programmed by Roger Lacroix              |
+-------------------------------------------------------------------+
|                      XMIT.CMD                                     |
|                                                                   |
|   Xmit.cmd is utility program to help the user SEND               |
|   or RECEIVE files to/from either VM CMS or MVS TSO.              |
|                                                                   |
|   Input Parameters:  NONE                                         |
|   Output Parameters: NONE                                         |
|                                                                   |
|   Input File:  username.DTA, username.LST                         |
|   Output File: username.DTA                                       |
|                                                                   |
+-------------------------------------------------------------------+
| Subroutines (and functions) and their uses:                       |
|                                                                   |
|  GET_USER_DATA                                                    |
|     Read the user's default parameters (username.DTA) located     |
|     in S:\XMIT\USER.                                              |
|                                                                   |
|  GETLU                                                            |
|     Ask the user which LU will be used during this transmission.  |
|                                                                   |
|  GETHOSTSTUFF                                                     |
|     Display the last 30 MEMORIZED host DSNs or Filename &         |
|     filetype then ask the user to select one or type 0 (zero) to  |
|     add a new DSN or filename & filetype.                         |
|                                                                   |
|  BUILDDATA                                                        |
|     Construct the DSN or filename & filetype from either the      |
|     MEMORIZED items or from the new data entered by the user.     |
|                                                                   |
|  SELECTED                                                         |
|     Determine what type of host name the user has selected.       |
|                                                                   |
|  PROPERPATH                                                       |
|     Go either to MVS or VM.                                       |
|                                                                   |
|  VM                                                               |
|     Get filename, filetype & filemode from the MEMORIZED list.    |
|                                                                   |
|  MVS                                                              |
|     Get DSN from the MEMORIZED list.                              |
|                                                                   |
|  GETDSNMEM                                                        |
|     Get DSN member name from the user.                            |
|                                                                   |
|  GETPCDIR                                                         |
|     Ask the user for the directory name.                          |
|                                                                   |
|  GETPCFILENAME                                                    |
|     Ask the user for the PC filename.                             |
|                                                                   |
|  VERIFYPCFILE                                                     |
|     Verfiy that the PC filename exists in the specified directory |
|                                                                   |
|  TRANSMISSION                                                     |
|     Send or Receive the specified PC filename to or from VM or MVS|
|                                                                   |
|  READ_DIRFILE                                                     |
|     If the user specified '*' as part of the PC filename, then    |
|     get the PC filenames in the particular sub-directory.         |
|                                                                   |
|                                                                   |
|                                                                   |
|  FINISHUP                                                         |
|     Check the return code of either the Send or Receive function  |
|     and if zero then MEMORIZE the latest host item.               |
|                                                                   |
|  REBUILDLIST                                                      |
|     Rebuild the MEMORIZED list of host names.                     |
|                                                                   |
|  WRITE_USER_DATA:                                                 |
|     Write new user parameters (default values & MEMORIZED items.) |
|                                                                   |
|  SPLIT_TOP:                                                       |
|     Display the top half of the MEMORIZED items panel.            |
|                                                                   |
|  SPLIT_BOTTOM:                                                    |
|     Display the bottom half of the MEMORIZED items panel.         |
|                                                                   |
|  MAINPANEL:                                                       |
|     Display main menu panel and ask the user for input.           |
|                                                                   |
|  PANEL:                                                           |
|     Display regular panel to the user and ask for input.          |
|                                                                   |
|  TABLE:                                                           |
|     Initialize regular messages and error messages.               |
|                                                                   |
|  MUSIC:                                                           |
|     Play a little music.                                          |
|                                                                   |
+-------------------------------------------------------------------+*/
PARSE UPPER ARG command junk
IF command \= ''  THEN  DO
   SAY 'No input parameter(s) are required.'
   SAY 'Example: XMIT.cmd '
   RETURN
END
ELSE NOP
/*+-------------------------------------+
  | don't let the user see the commands |
  +-------------------------------------+*/
'CLS'
'@ECHO OFF'
/*+-----------------------+
  | Initialize variables  |
  +-----------------------+*/
CALL TABLE
/*  env = 'OS2ENVIRONMENT' */
env = 'ENVIRONMENT'
username =  VALUE('USERNAME',,env)
username =  TRANSLATE(STRIP(username))    /* upper case & remove blanks */
user_dir = 'S:\XMIT\USER\'
fn_user_data = user_dir''username'.DTA'
fn_dirfile   = user_dir''username'.LST'
normal_msg = 'Y'
pcfile = ''
fn = ''
ft = ''
fm = ''
dsn = ''
dsnmembername = ''
quotes = 'N'
sequential = 'N'
file_alter = 'N'

invalid_chars    = "~`!%^&()=+|\/{}[];:<>,?"     /* invalid characters */
invalid_pc_chars = "~`!%^&()=|\/{}[];:<>,?"      /* invalid characters */

/*+--------------------------------------+
  | Get user data & set user defaults    |
  +--------------------------------------+*/
CALL GET_USER_DATA
IF rcode = 99 THEN DO       /* no userfile */
   CALL PANEL 1,emsg.13
   EXIT
END
ELSE NOP
pointer = 1
userinput = oldinput.pointer
luid = defaultlu
worksub = fn_worksub
/*+-----------------------------+
  | Go follow the proper path.  |
  +-----------------------------+*/
CALL SELECTED
CALL PROPERPATH

/*+------------------------------------------+
  | determine the range of LUs for this user |
  +------------------------------------------+*/
luletter = 'A B C D E F G H I J K L'
lowlu = WORD(luletter,1)
maxlu = WORD(luletter,totallu)
CALL MUSIC
/*+------------+
  |  Mainline  |
  +------------+*/
rcode = 0
DO WHILE (rcode = 0)
   CALL MAINPANEL
   PULL option fastpath
   option = TRANSLATE(STRIP(option))
   fastpath = TRANSLATE(STRIP(fastpath))
   SELECT
      WHEN option = '1' THEN DO
         luid = fastpath
         CALL GETLU
      END
      WHEN option = '2' THEN DO
         IF fastpath = ''  THEN
            CALL GETHOSTSTUFF
         ELSE DO
            userinput = fastpath
            pointer = 0
            quotes = 'N'
            sequential = 'N'
            dsn = ''
            dsnmembername = ''
            CALL SELECTED
            CALL PROPERPATH
         END
      END
      WHEN option = '3' THEN DO
         worksub = fastpath
         CALL GETPCDIR
      END
      WHEN option = '4' THEN DO
         pcfile = fastpath
         CALL GETPCFILENAME
      END
      WHEN option = 'R' | option = 'S' THEN DO
         IF option = 'R'  THEN
            command = 'RECEIVE'
         ELSE
            command = 'SEND'
         CALL VERIFYPCFILE
         IF pcfile_exist = 'Y' THEN DO
            /* if 1st MEMORIZED item then no need to rebuild the list */
            IF pointer = '1' THEN DO
               IF file_alter = 'Y' THEN DO  /* Did any options change? */
                  CALL WRITE_USER_DATA
               END
               ELSE NOP
            END
            ELSE DO   /* here, then pointer is any number other than 1 */
               CALL REBUILDLIST
               CALL WRITE_USER_DATA
            END
            CALL TRANSMISSION
            CALL FINISHUP
         END
         ELSE NOP
      END
      WHEN option = 'E' THEN DO
         /* if 1st MEMORIZED item then no need to rebuild the list */
         IF pointer \= '1' THEN
            CALL REBUILDLIST
         ELSE NOP
         CALL WRITE_USER_DATA
         rcode = 9
      END
      WHEN option = 'Q' THEN DO
         rcode = 9
      END
      OTHERWISE  DO
         err_msg = emsg.14''option
         normal_msg = 'N'
      END
   END /* select */
END

CALL PANEL  1,msg.17

RETURN

/*+--------------------------------------------------------------+
  | Subroutine to read user data file for default parameters     |
  +--------------------------------------------------------------+*/
GET_USER_DATA:

IF LINES(fn_user_data) = 0 THEN DO
   rcode = 99            /* no file */
   RETURN
END
ELSE NOP

/*+-----------------------------------------------+
  | retrieve defaults that were previously saved. |
  +-----------------------------------------------+*/
fn_worksub = LINEIN(fn_user_data)
pcfile = LINEIN(fn_user_data)
totallu = LINEIN(fn_user_data)
defaultlu = LINEIN(fn_user_data)
dsnmembername = LINEIN(fn_user_data)
sequential = LINEIN(fn_user_data)
quotes = LINEIN(fn_user_data)

/*+---------------------------------+
  | retrieve MEMORIZED host names   |
  +---------------------------------+*/
oldinput. = ''
i = 1
DO UNTIL LINES(fn_user_data) = 0
   oldinput.i = LINEIN(fn_user_data)
   IF oldinput.i \= ''  THEN
      i = i + 1
END
nothing = LINEOUT(fn_user_data)       /* close the file  */
oldinput.0 = i - 1
/*+--------------------------------------------+
  |  we will only allow 30 previous host names |
  +--------------------------------------------+*/
IF oldinput.0 > 30  THEN
   oldinput.0 = 30
ELSE
   NOP

RETURN

/*+-----------------------------------+
  |  Subroutine to get host LU letter |
  +-----------------------------------+*/
GETLU:

file_alter = 'Y'
IF (luid \= '' & (luid < lowlu | luid > maxlu | LENGTH(luid) > 1)) THEN DO
   err_msg = emsg.1''luid
   normal_msg = 'N'
   luid = ''
END
ELSE NOP
/*+----------------------------------------------+
  | Ask the user which LU s/he is logged onto.   |
  +----------------------------------------------+*/
DO WHILE (luid < lowlu | luid > maxlu | LENGTH(luid) > 1)

   CALL PANEL  10,msg.2,' ',' ',COPIES('-',70),' ',msg.3'[A - 'LEFT(maxlu,1)']',' ','OR',' ',msg.9'"'defaultlu'"'
   PULL luid
   luid = STRIP(luid)
   IF luid = '' THEN
      luid = defaultlu
   ELSE NOP
   IF (luid < lowlu | luid > maxlu | LENGTH(luid) > 1) THEN DO
      err_msg = emsg.1''luid
      normal_msg = 'N'
      luid = ''
   END
   ELSE NOP
END

RETURN

/*+--------------------------------------------------------------+
  | Subroutine to get host information.                          |
  | i.e. DSN(member-name) or filename, filetype and filemode     |
  +--------------------------------------------------------------+*/
GETHOSTSTUFF:

oldpointer = pointer
pointer = ''
CALL SPLIT_TOP  msg.13
DO WHILE  (pointer = '')
   /*+--------------------------------------------------------------+
     |   display panel so the user can choose an item from the list |
     +--------------------------------------------------------------+*/
   DO i=1 TO 15
      j = i + 15
      SAY '|'RIGHT(i,3)' 'LEFT(oldinput.i,34)'|'RIGHT(j,3)' 'LEFT(oldinput.j,34)'|'
   END
   CALL SPLIT_BOTTOM  msg.14,msg.15
   PULL pointer junk
   pointer = TRANSLATE(STRIP(pointer))   /* upper case & remove blanks */
   /*+-------------------------------------------------+
     |   validate the number which the user inputted.  |
     +-------------------------------------------------+*/
   IF (pointer < '0' | pointer > oldinput.0 ) THEN DO
      IF pointer = 'Q'  THEN DO
        pointer = oldpointer
        RETURN
      END
      ELSE  NOP
        CALL SPLIT_TOP  emsg.7''pointer
        CALL MUSIC
        pointer = ''
   END
   ELSE  NOP
END
CALL BUILDDATA

RETURN

/*+--------------------------------------------------------------+
  | Construct data input either from the user or from the list.  |
  +--------------------------------------------------------------+*/
BUILDDATA:

/*+---------------------------+
  |    reset variables        |
  +---------------------------+*/
fn = ''
ft = ''
fm = ''
userinput = ''
dsn = ''
dsnmembername = ''
sequential = 'N'
quotes = 'N'
file_alter = 'Y'
IF (pointer = '0') THEN DO
   /*+----------------------------------------------------------------+
     | Display panel: the user wants to enter new data. The user will |
     | enter either dataset name or filename, filetype [filemode]     |
     +----------------------------------------------------------------+*/
   DO WHILE userinput = ''
      CALL PANEL 5,msg.4,' ','OR',' ',msg.5
      PULL userinput
      userinput = STRIP(userinput)
      SELECT
         WHEN userinput = '' THEN DO
            err_msg = emsg.12
            normal_msg = 'N'
            userinput = ''
         END
         WHEN WORDS(userinput) >  '3' THEN DO
            err_msg = emsg.12''userinput
            normal_msg = 'N'
            userinput = ''
         END
         WHEN VERIFY(userinput,invalid_chars,M) \= 0  THEN DO
            err_msg = emsg.11''STRIP(LEFT(userinput,50))''
            normal_msg = 'N'
            userinput = ''
         END
         OTHERWISE  NOP
      END   /* select */
   END
END
ELSE  DO
   /*+---------------------------------------------------------+
     | The user has selected an item from the displayed list.  |
     +---------------------------------------------------------+*/
   userinput = oldinput.pointer
   CALL SELECTED
END

CALL PROPERPATH

RETURN

/*+---------------------------------------------------------+
  | Go follow the proper path.                              |
  +---------------------------------------------------------+*/
PROPERPATH:

IF WORD(userinput,2) \= '' THEN
   CALL VM
ELSE
   CALL MVS

RETURN

/*+---------------------------------------------------------+
  | The user has selected an item from the displayed list.  |
  +---------------------------------------------------------+*/
SELECTED:

IF POS('()',userinput) \= 0  THEN DO         /* MVS DSN(member)   */
   PARSE VALUE userinput WITH userinput'('tempmember')'the_rest
   userinput = userinput''the_rest
   IF tempmember \= '' THEN
      dsnmembername = tempmember
   ELSE NOP
END
ELSE
   IF WORD(userinput,2) = '' & fastpath = ''  THEN
      sequential = 'Y'                  /* sequenital file  */
   ELSE
      NOP             /* must be fn ft fm for VM or fastpath string */

RETURN

/*+-------------------------------------+
  | Subroutine to setup VM  parameters. |
  | Filename, filetype [,filemode]      |
  +-------------------------------------+*/
VM:

fn = WORD(userinput,1)
ft = WORD(userinput,2)
IF  WORD(userinput,3) = '' THEN
    fm = 'A'
ELSE
    fm = WORD(userinput,3)
show_host = fn' 'ft' 'fm

RETURN

/*+--------------------------------------------------------------+
  | Get MVS information:                                         |
  | Either  Dataset name and member-name or Sequential file name |
  +--------------------------------------------------------------+*/
MVS:

dsn = WORD(userinput,1)
/*+--------------------------------------------------------------+
  | if the user selected a sequential file from the list then    |
  | do not prompt the user for the member-name.                  |
  +--------------------------------------------------------------+*/
IF sequential = 'N' & dsnmembername = '' THEN  /* keep this if statement */
   CALL GETDSNMEM
ELSE
   NOP

IF SUBSTR(dsn,1,1) = "'" THEN DO       /* starts with a quote i.e. ' */
   PARSE VALUE dsn WITH "'"dsn"'"
   quotes = 'Y'
END
ELSE
   IF SUBSTR(dsn,1,1) = '"' THEN DO    /* starts with a quote  i.e. " */
      PARSE VALUE dsn WITH '"'dsn'"'
      quotes = 'Y'
   END
   ELSE
      quotes = 'N'

IF sequential = 'Y' THEN
   show_host = dsn
ELSE  DO
   show_host = dsn"("dsnmembername")"
END
IF quotes = 'Y' THEN
   show_host = "'"show_host"'"
ELSE
   NOP

RETURN

/*+------------------------------------------------------------------+
  | get dataset member name or determine if this is a sequential file|
  +------------------------------------------------------------------+*/
GETDSNMEM:

dsnmembername = '***'
DO WHILE (LENGTH(dsnmembername) > 8 |  (dsnmembername \= '' & SUBSTR(dsnmembername,1,1) < 'A'))
   /*+---------------------------------------------------------------------------+
     | Display panel to get member name or if blank then dsn is a sequential file|
     +---------------------------------------------------------------------------+*/
   CALL PANEL 5,msg.6,' ','OR',' ',msg.7
   PULL dsnmembername
   dsnmembername = STRIP(dsnmembername)
   /*+-------------------------------------------------+
     | Validate dsnmembername which the user inputted. |
     +-------------------------------------------------+*/
   SELECT
      WHEN dsnmembername = '' THEN DO
         sequential = 'Y'
      END
      WHEN VERIFY(dsnmembername,invalid_chars,M) \= 0  THEN DO
         err_msg = emsg.11''STRIP(LEFT(dsnmembername,20))''
         normal_msg = 'N'
         dsnmembername = '***'
      END
      WHEN LENGTH(dsnmembername) > 8 THEN  DO
         err_msg = emsg.3''dsnmembername
         normal_msg = 'N'
         dsnmembername = '***'
      END
      WHEN (dsnmembername \= '' & SUBSTR(dsnmembername,1,1) < 'A') THEN DO
         err_msg = emsg.4''dsnmembername
         normal_msg = 'N'
         dsnmembername = '***'
      END
      OTHERWISE  NOP           /* if here then we have dsn member-name */
   END   /* select */
END

RETURN

/*+-------------------------------------+
  | Subroutine to get pc sub-directory  |
  +-------------------------------------+*/
GETPCDIR:

file_alter = 'Y'
IF worksub \= '' THEN DO
   /*   remove last "\" */
   IF LENGTH(worksub) > 3 & SUBSTR(worksub,LENGTH(worksub),1) = '\'  THEN
      worksub = SUBSTR(worksub,1,LENGTH(worksub)-1)
   ELSE
      NOP
   curdir = DIRECTORY()          /* save current sub directory  */
   newdir = DIRECTORY(worksub)   /* set new directory  */
   IF newdir = '' THEN  DO
      err_msg = emsg.8''STRIP(LEFT(worksub,50))''
      normal_msg = 'N'
      worksub = ''
   END
   ELSE NOP
   newdir = DIRECTORY(curdir)    /* reset dir. to old directory */
END
ELSE NOP

DO WHILE worksub = ''
   /*+---------------------------------------------+
     | Display panel to get PC path name.          |
     | Supply the user with a default PC path name.|
     +---------------------------------------------+*/
   CALL PANEL 5,msg.10,' ','OR',' ',msg.9'"'STRIP(LEFT(fn_worksub,50))'"'
   PULL worksub
   worksub = STRIP(worksub)
   IF worksub = '' THEN
      worksub = fn_worksub
   ELSE
      NOP
   /*   remove last "\" */
   IF LENGTH(worksub) > 3 & SUBSTR(worksub,LENGTH(worksub),1) = '\'  THEN
      worksub = SUBSTR(worksub,1,LENGTH(worksub)-1)
   ELSE
      NOP
   curdir = DIRECTORY()          /* save current sub directory  */
   newdir = DIRECTORY(worksub)   /* set new directory  */
   IF newdir = '' THEN  DO
      err_msg = emsg.8''STRIP(LEFT(worksub,50))''
      normal_msg = 'N'
      worksub = ''
   END
   ELSE NOP
   newdir = DIRECTORY(curdir)    /* reset dir. to old directory */
END
/*+-------------------------------------------+
  | make sure path has the proper backslashes.|
  +-------------------------------------------+*/
IF SUBSTR(worksub,LENGTH(worksub),1) \= '\'  THEN
   worksub = worksub'\'
ELSE
   NOP

RETURN

/*+-----------------------------------+
  | Subroutine to get pc filename     |
  +-----------------------------------+*/
GETPCFILENAME:
file_alter = 'Y'

/*+----------------------------------------------------+
  | verify fastpath  PC filename                       |
  +----------------------------------------------------+*/
SELECT
   WHEN POS('*',pcfile) \= 0 & POS('.',pcfile) \=0 &,
        POS('*',pcfile) > POS('.',pcfile)  THEN  DO
      err_msg = emsg.5
      normal_msg = 'N'
      pcfile = ''
   END
   WHEN POS('*',pcfile,POS('*',pcfile)+1) \= 0 THEN  DO
      err_msg = emsg.6
      normal_msg = 'N'
      pcfile = ''
   END
   WHEN VERIFY(pcfile,invalid_pc_chars,M) \= 0 THEN DO
      err_msg = emsg.11''STRIP(LEFT(pcfile,50))''
      normal_msg = 'N'
      pcfile = ''
   END
   OTHERWISE NOP
END   /* select */

/*+----------------------------------------------------+
  | Supply a default PC filename for the user.         |
  | Either filename from VM or dsnmembername from MVS. |
  +----------------------------------------------------+*/
SELECT
   WHEN fn \= '' THEN
      temp = fn
   WHEN dsnmembername \= '' THEN
      temp = dsnmembername
   OTHERWISE
      temp = ''
END   /* select */

DO WHILE pcfile = ''
   /*+---------------------------------------------+
     | Display panel to get PC filename.           |
     | Supply the user with a default PC filename. |
     +---------------------------------------------+*/
   IF temp \= '' THEN
      CALL PANEL 5,msg.8,' ','OR',' ',msg.9'"'STRIP(LEFT(temp,8))'"'
   ELSE
      CALL PANEL 1,msg.8
   PULL pcfile
   pcfile = STRIP(pcfile)
   SELECT
      WHEN POS('*',pcfile) \= 0 & POS('.',pcfile) \=0 &,
           POS('*',pcfile) > POS('.',pcfile)  THEN  DO
         err_msg = emsg.5
         normal_msg = 'N'
         pcfile = ''
      END
      WHEN POS('*',pcfile,POS('*',pcfile)+1) \= 0 THEN  DO
         err_msg = emsg.6
         normal_msg = 'N'
         pcfile = ''
      END
      WHEN VERIFY(pcfile,invalid_pc_chars,M) \= 0 THEN DO
         err_msg = emsg.11''STRIP(LEFT(pcfile,50))''
         normal_msg = 'N'
         pcfile = ''
      END
      WHEN pcfile = '' & temp = ''  THEN  DO
         err_msg = emsg.10
         normal_msg = 'N'
         pcfile = ''
      END
      OTHERWISE DO
        IF pcfile = '' & temp \= '' THEN
           pcfile = temp
        ELSE
           NOP
      END
   END   /* select */
END

RETURN

/*+--------------------------------------+
  | Subroutine to verfiy pc filename     |
  +--------------------------------------+*/
VERIFYPCFILE:

pcfile_exist = 'Y'
IF POS('*+',pcfile) = 0  THEN  DO
   IF (command = 'SEND' | (command = 'RECEIVE' & POS('*',pcfile) \= 0)) THEN DO
      'DIR 'worksub''pcfile' /F  > 'fn_dirfile
      IF RC = 0  THEN
         NOP    /* do nothing */
      ELSE  DO
         err_msg = emsg.9''pcfile
         normal_msg = 'N'
         pcfile_exist = 'N'
      END
/*      nothing = LINEOUT(fn_dirfile)        close the file  */
   END
   ELSE NOP
END
ELSE NOP

RETURN

/*+----------------------------------------------+
  | transit or receive data to and from the host |
  +----------------------------------------------+*/
TRANSMISSION:

IF POS('*',pcfile) \= 0 THEN  DO
   CALL READ_DIRFILE
END
ELSE DO
   dirlist.1 = pcfile
   dirlist.0 = 1
END
DO i=1 to dirlist.0
   PARSE VALUE dirlist.i WITH pctemp'.'the_rest
   pctemp = STRIP(pctemp)
   IF dsn \= '' THEN  DO
      IF sequential = 'Y' THEN
         xmithost = dsn
      ELSE  DO                        /* DSN(member)      */
         IF POS('*',pcfile) \= 0 THEN
            xmithost = dsn"("pctemp")"
         ELSE
            xmithost = dsn"("dsnmembername")"
      END
      xmit_options = " ASCII CRLF T(01)"
      IF quotes = 'Y' THEN
         xmithost = "'"xmithost"' "
      ELSE
         xmithost = xmithost" "
   END
   ELSE    DO                        /* fn ft fm         */
      IF command = 'RECEIVE'  THEN
         xmit_options = " (ASCII CRLF T(01)"
      ELSE
         xmit_options = " (ASCII CRLF LRECL 80 T(01)"
      IF POS('*',pcfile) \= 0 THEN
         xmithost =pctemp' 'ft' 'fm
      ELSE
         xmithost =fn' 'ft' 'fm
   END

   DO 2
      SAY ' '
   END
   SAY 'If you need to cancel this transmission, press Control-C once.'
   SAY 'Aborting this transmission can take up to 30 seconds.'
   DO 2
      SAY ' '
   END

   SAY ""command worksub""dirlist.i luid":"xmithost""xmit_options
   ""command worksub""dirlist.i luid":"xmithost""xmit_options
   rcode2 = RC
END

RETURN

/*+----------------------------------------------------+
  | get directory listing of pc transmission directory |
  +----------------------------------------------------+*/
READ_DIRFILE:

nothing = LINEIN(fn_dirfile,1,0)  /* reset pointer in the file */
i = 1
DO UNTIL LINES(fn_dirfile) = 0
   dirlist.i = LINEIN(fn_dirfile)
   IF dirlist.i \= '' & POS('\.',dirlist.i) \= 0  THEN
      NOP
   ELSE DO
      dirlist.i = FILESPEC("NAME",dirlist.i)   /* get pc file name */
      i = i + 1
   END
END
nothing = LINEOUT(fn_dirfile)       /* close the file  */
dirlist.0 = i - 1

RETURN

/*+-----------------------------------------------+
  | Subroutine to cleanup after the transmission  |
  +-----------------------------------------------+*/
FINISHUP:

  IF rcode2 > 1 THEN  DO
     /*+------------------------------------------------------+
       | An error occurred during the transmission            |
       | rc=1  - Some files truncated                         |
       | rc=22 - keyboard inhibited, file transfer cancelled  |
       | rc=24 - File transfer cancelled                      |
       +------------------------------------------------------+*/
     SELECT
     WHEN rcode2 = '1' THEN DO
        err_msg = emsg.15
        normal_msg = 'N'
     END
     WHEN rcode2 = '22' THEN DO
        err_msg = emsg.16
        normal_msg = 'N'
     END
     WHEN rcode2 = '24' THEN DO
        err_msg = emsg.17
        normal_msg = 'N'
     END
     OTHERWISE DO
        err_msg = emsg.17
        normal_msg = 'N'
     END
     END /* select */
  END
  ELSE  DO
     /*+----------------------------+
       | the transmission was O.K.  |
       +----------------------------+*/
     err_msg = emsg.19     /* successful msg */
     normal_msg = 'N'
  END

  'ERASE 'fn_dirfile                 /* erase the file for this user */
RETURN

/*+---------------------------------------------------------------+
  | Rebuild the MEMORIZED list of host names.                     |
  +---------------------------------------------------------------+*/
REBUILDLIST:

IF pointer \= '0' THEN DO
   /*+---------------------------------------------+
     |  The user selected an item from the list,   |
     |  Therefore remove it from the list          |
     +---------------------------------------------+*/
   DO j=pointer to oldinput.0-1
      k = j + 1
      oldinput.j = oldinput.k
   END
   oldinput.oldinput.0 = ' '
   oldinput.0 = oldinput.0 - 1
END
ELSE NOP
/*+---------------------------------------------------+
  |  move the list entries down one position and      |
  |  add the transmission item to the top of the list |
  +---------------------------------------------------+*/
DO j=oldinput.0+1 to 2 by -1
   k = j -1
   oldinput.j = oldinput.k
END
IF  ft \= '' THEN
    oldinput.1 = LEFT(fn,9)''LEFT(ft,9)''LEFT(fm,4)
ELSE  DO
    IF dsnmembername \= '' THEN
       oldinput.1 = dsn"()"
    ELSE
       oldinput.1 = dsn
    IF quotes = 'Y' THEN
       oldinput.1 = "'"oldinput.1"'"
    ELSE
       NOP
END
oldinput.0 = oldinput.0 + 1
pointer = 1

RETURN

/*+---------------------------------------------------------------+
  | Save the list of old host file names for future use.          |
  +---------------------------------------------------------------+*/
WRITE_USER_DATA:

file_alter = 'N'
/*+----------------------------------+
  | tell the user what we are doing  |
  +----------------------------------+*/
CALL PANEL  1,msg.16

/*+-----------------------------------------+
  |  we will only store  a max. of 30 items |
  +-----------------------------------------+*/
IF oldinput.0 > 30  THEN
   oldinput.0 = 30
ELSE
   NOP
/*+-----------------------+
  |  get rid of old files |
  +-----------------------+*/
'ATTRIB  -R 'fn_user_data          /* remove read only attribute   */
'ERASE 'fn_user_data               /* erase the file for this user */
/*+-----------------------------------+
  |  write user input list to a file  |
  +-----------------------------------+*/
nothing = LINEIN(fn_user_data,1,0)  /* reset pointer in the file */

nothing = LINEOUT(fn_user_data,worksub)
nothing = LINEOUT(fn_user_data,pcfile)
nothing = LINEOUT(fn_user_data,totallu)
nothing = LINEOUT(fn_user_data,luid)

nothing = LINEOUT(fn_user_data,dsnmembername)
nothing = LINEOUT(fn_user_data,sequential)
nothing = LINEOUT(fn_user_data,quotes)

DO i=1 to oldinput.0
   nothing = LINEOUT(fn_user_data,oldinput.i)
END
nothing = LINEOUT(fn_user_data)       /* close the file  */
'ATTRIB  +R 'fn_user_data' '          /* mark the file as read only */

RETURN

/*+----------------------------+
  | Top display of split table |
  +----------------------------+*/
SPLIT_TOP:
PARSE ARG p.1

"CLS"
SAY '+'COPIES('-',77)'+'
SAY '|  'LEFT(p.1,75)'|'
SAY '+'COPIES('-',38)'+'COPIES('-',38)'+'

RETURN

/*+-------------------------------+
  | Bottom display of split table |
  +-------------------------------+*/
SPLIT_BOTTOM:
PARSE ARG p.1,p.2

SAY '+'COPIES('-',38)'+'COPIES('-',38)'+'
SAY '|  'LEFT(p.1,75)'|'
SAY '|  'LEFT(p.2,75)'|'
SAY '+'COPIES('-',77)'+'

RETURN

/*+--------------------------------------+
  | Display Panel with user information. |
  +--------------------------------------+*/
PANEL:
PARSE ARG num,p.1,p.2,p.3,p.4,p.5,p.6,p.7,p.8,p.9,p.10,p.11,p.12

"CLS"
IF normal_msg = 'Y' THEN DO
   SAY '+'COPIES('-',70)'+'
END
ELSE DO
   CALL MUSIC
   SAY '+'COPIES('-',70)'+'
   SAY '|  'LEFT(err_msg,68)'|'
   SAY '+'COPIES('-',70)'+'
   err_msg = ' '
END

SAY '|'COPIES(' ',70)'|'
SAY '|'COPIES(' ',70)'|'
DO ii=1 to num
   SAY '|'CENTER(p.ii,70)'|'
END
SAY '|'COPIES(' ',70)'|'
SAY '|'COPIES(' ',70)'|'
SAY '+'COPIES('-',70)'+'
normal_msg = 'Y'

RETURN


/*+--------------------------------------+
  | Display Panel with user information. |
  +--------------------------------------+*/
MAINPANEL:
PARSE ARG p.1

"CLS"
IF normal_msg = 'Y' THEN DO
   SAY '++'COPIES('-',69)'++'
   SAY '||'CENTRE('Welcome to Xmit.',69)'||'
   SAY '|+-----+'COPIES('-',63)'+|'
END
ELSE DO
   CALL MUSIC
   SAY '++'COPIES('-',69)'++'
   SAY '||  'LEFT(err_msg,67)'||'
   SAY '|+-----+'COPIES('-',63)'+|'
   err_msg = ' '
END

SAY '||  1  |  'LEFT('Host LU is: 'STRIP(LEFT(luid,5)),61)'||'
SAY '||     |  'COPIES(' ',61)'||'
SAY '||  2  |  'LEFT('Host DSN name or Filename, Filetype & Filemode:',61)'||'
SAY '||     |  'LEFT(show_host,61)'||'
SAY '||     |  'COPIES(' ',61)'||'
SAY '||  3  |  'LEFT('PC Work Sub-directory:',61)'||'
SAY '||     |  'LEFT(worksub,61)'||'
SAY '||     |  'COPIES(' ',61)'||'
SAY '||  4  |  'LEFT('PC Filename is: 'STRIP(LEFT(pcfile,15)),61)'||'

SAY '|+-----+'COPIES('-',63)'+|'
SAY '||  R  |  'LEFT('To Receive a file from the host system',61)'||'
SAY '||  S  |  'LEFT('To Send a file to the host system',61)'||'
SAY '|+-----+'COPIES('-',63)'+|'
SAY '||  E  |  'LEFT('Exit this program and save the parameters.',61)'||'
SAY '||  Q  |  'LEFT('Quit this program without saving the parameters.',61)'||'
SAY '|+-----+'COPIES('-',63)'+|'
SAY '||  'LEFT('Choose a number {1, 2, 3, or 4} or a letter {E, Q, R or S}',67)'||'
SAY '++'COPIES('-',69)'++'

normal_msg = 'Y'

RETURN

/*+-------------------------------------+
  | Music                               |
  +-------------------------------------+*/
MUSIC:

CALL BEEP 50,120

RETURN

/*+-----------------------------------------------+
  | Information and error message table           |
  +-----------------------------------------------+*/
TABLE:

msg.1 = 'You wish to transmit or receive a file.'
msg.2 = 'Please be sure that you are logged on to the proper host session.'
msg.3 = 'Which LU is connected to the host session??   '
msg.4 = 'For MVS, please type the Dataset name'
msg.5 = 'For VM,  type the filename, filetype, [filemode].'
msg.6 = 'Please type the MVS Dataset MEMBER-NAME'
msg.7 = 'Press ENTER if this is a sequential file.'
msg.8 = 'Please specify the PC file name and extension.'
msg.9 = 'Press ENTER to use '
msg.10 = 'Please specify the Path to the file'
msg.11 = 'An error occurred, press the ENTER to try again'
msg.12 = 'Type END, CANCEL or QUIT to quit.'
msg.13 = 'Please choose one of the numbers listed below.'
msg.14 = ''RIGHT('0',3)'  For NEW data to be specified.'
msg.15 = ''RIGHT('Q',3)'  Quit and return to main menu.'
msg.16 = 'Please wait ... writing parameters to your user file.'
msg.17 = 'Xmit has now ended.'
/*+-------------------------------------+
  | Error message table.                |
  +-------------------------------------+*/
emsg.1 = 'The LU specified is invalid. ==> '
emsg.2 = 'Invalid number. Please choose again.==> '
emsg.3 = 'Member name is larger than 8 characters. ==> '
emsg.4 = '1st character must be alphabetic character (non-numeric). ==> '
emsg.5 = 'Cannot use the wild card qualifier "*" after the period "."'
emsg.6 = 'Cannot use the wild card qualifier "*" TWICE!!!!!'
emsg.7 = 'Invalid number. Please choose again. ==> '
emsg.8 = 'The inputted directory does NOT exist: ==> '
emsg.9 = 'The file(s) does NOT exist in the directory: ==> '
emsg.10 = 'Please type a PC file name.'
emsg.11 = 'You have inputted invalid character(s): ==>'
emsg.12 = 'Invalid response. '
emsg.13 = 'You have no user data, please run the SETUP.CMD program. '
emsg.14 = 'ERROR - You selected an invalid a number or letter. ==> '
emsg.15 = 'ERROR - Some files truncated during the transmission. '
emsg.16 = 'ERROR - "Keyboard inhibited", the file transfer was cancelled.'
emsg.17 = 'ERROR - The file transfer was cancelled.'
emsg.18 = ''
emsg.19 = 'The transmission was successful. '

RETURN
